
import {_dbbyTableSymbol} from "./dbby-table-symbol.js"
import {DbbyRow, DbbyConditionLeaf} from "./dbby-types.js"

export function and<
		Row extends DbbyRow,
		C extends DbbyConditionLeaf<Row>[] = DbbyConditionLeaf<Row>[]
	>(...conditions: C): ["and", ...C] {
	return ["and", ...conditions]
}

export function or<
		Row extends DbbyRow,
		C extends DbbyConditionLeaf<Row>[] = DbbyConditionLeaf<Row>[]
	>(...conditions: C): ["or", ...C] {
	return ["or", ...conditions]
}

export function find<Row extends DbbyRow>(...rows: Partial<Row>[]) {
	return rows.length
		? {conditions: or(...rows.map(row => ({equal: row})))}
		: {conditions: false as const}
}

export function findAll<V, R extends DbbyRow = DbbyRow>(values: V[], valueForRow: (v: V) => Partial<R>) {
	return {conditions: or(...values.map(v => ({equal: valueForRow(v)})))}
}

export function isDbbyTable(x: any) {
	return x && typeof x === "object" && x[_dbbyTableSymbol]
}
