import { getComponents } from "./get-components.js";
import { assembleModels } from "./assemble-models.js";
import { setupModalSystem } from "./modal/setup-modal-system.js";
import { loginWithLinkTokenOrUseExistingLogin } from "./auth/login-with-link-token-or-use-existing-login.js";
import { AccessLoginExpiredError } from "../../features/auth/aspects/users/models/errors/access-errors.js";
export async function assembleAndInitializeFrontend({ appId, popups, storage, authMediator, remote, }) {
    const { modals, modalsElement } = setupModalSystem();
    const models = await assembleModels({
        appId,
        remote,
        popups,
        storage,
        authMediator,
    });
    const components = getComponents({ models, modals });
    await loginWithLinkTokenOrUseExistingLogin({
        accessModel: models.accessModel,
        link: window.location.toString(),
        onError: error => {
            if (error instanceof AccessLoginExpiredError)
                modals.alert({
                    title: "expired login link",
                    body: "this login link has expired, please try again",
                });
            else
                modals.alert({
                    title: "invalid login link",
                    body: "something is wrong with this login link, please try again",
                });
        },
        onDone: () => {
            window.location.hash = "";
        },
    });
    return { components, models, modals, modalsElement };
}
//# sourceMappingURL=assemble-and-initialize-frontend.js.map