import { objectMap } from "../../../../toolbox/object-map.js";
export function logAllCalls({ logger, fullyDebug, remote }) {
    let count = 1;
    function recurse(remote, path = []) {
        return objectMap(remote, (value, key) => {
            const currentPath = [...path, key];
            if (typeof value === "function") {
                return async (...args) => {
                    const callNumber = count++;
                    const label = `#${callNumber} ${currentPath.join(".")}`;
                    if (fullyDebug)
                        logger.log(label, ...args);
                    else
                        logger.log(label);
                    const result = await value(...args);
                    if (fullyDebug)
                        logger.log(`    #${callNumber} return:`, result);
                    return result;
                };
            }
            else if (typeof (value ?? undefined) === "object")
                return recurse(value, currentPath);
            else
                throw new Error("unknown remote type");
        });
    }
    return recurse(remote);
}
//# sourceMappingURL=log-all-calls.js.map