import { ApiError } from "renraku/x/api/api-error.js";
export function makePrivilegeChecker(permit, privileges) {
    function hasPrivilege(key) {
        return permit.privileges.includes(privileges[key]);
    }
    function requirePrivilege(key) {
        if (!hasPrivilege(key))
            throw new ApiError(403, `forbidden; privilege required "${key}"`);
    }
    function requireNotHavePrivilege(key) {
        if (hasPrivilege(key))
            throw new ApiError(403, `forbidden: must not have privilege "${key}"`);
    }
    return {
        hasPrivilege,
        requirePrivilege,
        requireNotHavePrivilege,
    };
}
//# sourceMappingURL=make-privilege-checker.js.map